/*
Die Korrelationsfunktion wird hier durch Multiplikation
der Datenpunkte (z.B. 2048 Werte) von Kanal 1 und Kanal 2 
sowie anschließendem Aufsummieren der Ergebnisse berechnet.
Man erhält damit den Wert der Korrelationsfunktion
an der Stelle 0, also ohne Verschiebung.
Nach einem Durchlauf wird der Datensatz von
Kanal 2 um einen Schritt nach rechts zu größeren Zeiten
verschoben. Alle Daten die am oberen Ende 'herausgeschoben' 
werden, werden am unteren Ende wieder 'hereingeschoben'.
Nun erfolgt erneut ein Multiplizieren der übereinander liegenden
Kanalwerte sowie ein Aufsummieren der Ergebnisse.
Man erhält den Wert der Korrelationsfunktion an der Stelle 1,
also bei Verschiebung um eine Zeiteinheit, oder einen
Arraywert.
*/

void 
TABELLE_xcorr(TABELLE *tab
              , int index_sp1
              , int index_sp2
              , int index_spziel)
{
int i,j;
int ring_size, ring_size_half;
int ring_pos;
double *column;

ring_size = tab->n_data_lines;
ring_size_half = rint(ring_size / 2);

column = (double*)malloc((ring_size+1)*sizeof(double));

for (i = 0; i < ring_size; i++)
   {
   for (j = 0; j < ring_size; j++)
      {
      ring_pos = (ring_size * (1 + abs((i+j)/ring_size)) + (i+j)) % ring_size;

      column[i]
          += (double)(  tab->column[index_sp1][j] 
                         *  tab->column[index_sp2][ring_pos]);
      }
      column[i] /= (double)(tab->n_data_lines); 
   }

/* 
Umformen der Korrelationsfunktion:
Bei Plot über Indexwerten von diesen jeweils 'ring_size_half'
abziehen, dann das Maximum suchen.
Anzahl Indexwerte am Maximum in positiver oder negativer Richtung,
multipliziert mit dem Messintervall liefert die Zeitdifferenz 
zwischen Signal 1 und 2.
*/
for (i=0; i<1024; i++)
   {
   tab->column[index_spziel][i+ring_size_half] = column[i];
   tab->column[index_spziel][i]      = column[i+ring_size_half];
   }
   
free(column);

}


int 
TABELLE_read (TABELLE *tab, unsigned char *sz_file)
{
FILE			*fp;
int 			ilen;
short 			c;
int 			str_size;
int 			i,j;
char 			*p;
int 			ipos, icomment;
unsigned char 	sz_string[4096];

fp = fopen(sz_file, "rb");
if (fp == NULL)
  {
  printf("Datei nicht gefunden.\n");
  return 1;
  }
tab->n_lines = 0;
while ((c=getc(fp))!=EOF)
     {
     if(c=='\n')
      tab->n_lines++;
     }
rewind(fp);

tab->n_data_lines     = tab->n_lines;
tab->n_columns        = 0;
tab->n_columns_prev   = 0;

for (i=0; i<tab->n_lines; i++)
   {
    fgets(sz_string,sizeof(sz_string),fp);
    str_size = strlen(sz_string);
    tab->n_columns = 0;
    if (sz_string[0] == '#')
      tab->n_data_lines--;
      else
          {
           for (j=0; j<str_size; j++)
              {
              if (sz_string[j]==';')
                tab->n_columns++;
              }
              if (tab->n_columns != tab->n_columns_prev && tab->n_columns_prev != 0)
                {
                printf ("Spaltenlänge uneinheitlich.\n");
                return 1;
                }
              tab->n_columns_prev = tab->n_columns;
              }
   }
rewind(fp);

for (i=0; i<=tab->n_columns; i++)
   tab->column[i] = (double*) malloc((tab->n_lines+1) * sizeof(double));

tab->n_comment_lines = tab->n_lines - tab->n_data_lines;
for (i=0; i<=tab->n_comment_lines; i++)
   tab->sz_comment_line[i] = (unsigned char*) malloc(4094);

ipos    =0;
icomment=0;
for (i=0; i<tab->n_lines; i++)
   {
    fgets (sz_string, sizeof(sz_string), fp);
    
    if (sz_string[0]!='#')
      {
      /* Trennzeichen */	      
      p = strtok(sz_string, ";");
      if (p) 
        {
        for (j=0; j <= tab->n_columns; j++)
           {
            tab->column[j][ipos] = strtod (p, NULL);
            p = strtok (NULL, ";");
           }
        }
      if (ipos < tab->n_data_lines)
        ipos++;   
      }
     else
        {
         tab->sz_comment_line[icomment] = malloc(sizeof(sz_string));
         strncpy(tab->sz_comment_line[icomment],sz_string,sizeof(sz_string));
         icomment++;
        }
   }
fclose(fp);
return 0;
}

